@@ -9,41 +9,42 @@
  * See LICENSE-mit for more information.
  *
  */
-
-#include <X11/Xlib-xcb.h>
-#include <assert.h>
 #include <limits.h>
-#include <pixman.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+
+#include <X11/Xlib-xcb.h>
+#include <pixman.h>
+#include <uthash.h>
 #include <xcb/composite.h>
 #include <xcb/xcb.h>
+#include <xcb/xcb_aux.h>
 
 #include "backend/backend.h"
 #include "backend/backend_common.h"
-#include "backend/gl/gl_common.h"
-#include "backend/gl/glx.h"
 #include "common.h"
 #include "compiler.h"
 #include "config.h"
 #include "log.h"
 #include "picom.h"
-#include "region.h"
-#include "utils.h"
-#include "win.h"
+#include "utils/misc.h"
 #include "x.h"
 
-struct _glx_pixmap {
-	GLXPixmap glpixmap;
-	xcb_pixmap_t pixmap;
-	bool owned;
-};
+#include "gl_common.h"
+#include "glx.h"
 
 struct _glx_data {
 	struct gl_data gl;
 	xcb_window_t target_win;
 	GLXContext ctx;
+	struct glx_fbconfig_cache *cached_fbconfigs;
+};
+
+struct glx_fbconfig_cache {
+	UT_hash_handle hh;
+	struct xvisual_info visual_info;
+	struct glx_fbconfig_info info;
 };
 
 #define glXGetFBConfigAttribChecked(a, b, attr, c)                                       \
@@ -54,28 +55,25 @@ struct _glx_data {
 		}                                                                        \
 	} while (0)
 
-struct glx_fbconfig_info *glx_find_fbconfig(struct x_connection *c, struct xvisual_info m) {
-	log_debug("Looking for FBConfig for RGBA%d%d%d%d, depth %d", m.red_size,
-	          m.blue_size, m.green_size, m.alpha_size, m.visual_depth);
+bool glx_find_fbconfig(struct x_connection *c, struct xvisual_info m,
+                       struct glx_fbconfig_info *info) {
+	log_debug("Looking for FBConfig for RGBA%d%d%d%d, depth: %d, visual id: %#x", m.red_size,
+	          m.blue_size, m.green_size, m.alpha_size, m.visual_depth, m.visual);
+
+	info->cfg = NULL;
 
 	int ncfg;
 	// clang-format off
 	GLXFBConfig *cfg =
 	    glXChooseFBConfig(c->dpy, c->screen, (int[]){
-	                          GLX_RENDER_TYPE, GLX_RGBA_BIT,
-	                          GLX_DRAWABLE_TYPE, GLX_PIXMAP_BIT,
-	                          GLX_X_VISUAL_TYPE, GLX_TRUE_COLOR,
-	                          GLX_X_RENDERABLE, true,
-	                          GLX_FRAMEBUFFER_SRGB_CAPABLE_EXT, (GLint)GLX_DONT_CARE,
-	                          GLX_BUFFER_SIZE, m.red_size + m.green_size +
-	                                           m.blue_size + m.alpha_size,
 	                          GLX_RED_SIZE, m.red_size,
-	                          GLX_BLUE_SIZE, m.blue_size,
 	                          GLX_GREEN_SIZE, m.green_size,
+	                          GLX_BLUE_SIZE, m.blue_size,
 	                          GLX_ALPHA_SIZE, m.alpha_size,
-	                          GLX_STENCIL_SIZE, 0,
-	                          GLX_DEPTH_SIZE, 0,
-	                          0
+	                          GLX_DRAWABLE_TYPE, GLX_PIXMAP_BIT,
+	                          GLX_X_RENDERABLE, True,
+	                          GLX_CONFIG_CAVEAT, GLX_NONE,
+	                          None,
 	                      }, &ncfg);
 	// clang-format on
 
@@ -114,7 +112,8 @@ struct glx_fbconfig_info *glx_find_fbconfig(struct x_connection *c, struct xvisu
 		int visual;
 		glXGetFBConfigAttribChecked(c->dpy, cfg[i], GLX_VISUAL_ID, &visual);
 		if (m.visual_depth != -1 &&
-		    x_get_visual_depth(c, (xcb_visualid_t)visual) != m.visual_depth) {
+		    xcb_aux_get_depth_of_visual(c->screen_info, (xcb_visualid_t)visual) !=
+		        m.visual_depth) {
 			// FBConfig and the correspondent X Visual might not have the same
 			// depth. (e.g. 32 bit FBConfig with a 24 bit Visual). This is
 			// quite common, seen in both open source and proprietary drivers.
@@ -143,39 +142,31 @@ struct glx_fbconfig_info *glx_find_fbconfig(struct x_connection *c, struct xvisu
 		min_cost = depthbuf + stencil + bufsize * (doublebuf + 1);
 	}
 	free(cfg);
-	if (!found) {
-		return NULL;
+	if (found) {
+		info->cfg = ret;
+		info->texture_tgts = texture_tgts;
+		info->texture_fmt = texture_fmt;
+		info->y_inverted = y_inverted;
 	}
-
-	auto info = cmalloc(struct glx_fbconfig_info);
-	info->cfg = ret;
-	info->texture_tgts = texture_tgts;
-	info->texture_fmt = texture_fmt;
-	info->y_inverted = y_inverted;
-	return info;
+	return found;
 }
 
 /**
  * Free a glx_texture_t.
  */
 static void glx_release_image(backend_t *base, struct gl_texture *tex) {
-	struct _glx_pixmap *p = tex->user_data;
+	GLXPixmap *p = tex->user_data;
 	// Release binding
-	if (p->glpixmap && tex->texture) {
+	if (p && tex->texture) {
 		glBindTexture(GL_TEXTURE_2D, tex->texture);
-		glXReleaseTexImageEXT(base->c->dpy, p->glpixmap, GLX_FRONT_LEFT_EXT);
+		glXReleaseTexImageEXT(base->c->dpy, *p, GLX_FRONT_LEFT_EXT);
 		glBindTexture(GL_TEXTURE_2D, 0);
 	}
 
 	// Free GLX Pixmap
-	if (p->glpixmap) {
-		glXDestroyPixmap(base->c->dpy, p->glpixmap);
-		p->glpixmap = 0;
-	}
-
-	if (p->owned) {
-		xcb_free_pixmap(base->c->c, p->pixmap);
-		p->pixmap = XCB_NONE;
+	if (p) {
+		glXDestroyPixmap(base->c->dpy, *p);
+		*p = 0;
 	}
 
 	free(p);
@@ -197,15 +188,17 @@ void glx_deinit(backend_t *base) {
 		gd->ctx = 0;
 	}
 
+	struct glx_fbconfig_cache *cached_fbconfig = NULL, *tmp = NULL;
+	HASH_ITER(hh, gd->cached_fbconfigs, cached_fbconfig, tmp) {
+		HASH_DEL(gd->cached_fbconfigs, cached_fbconfig);
+		free(cached_fbconfig);
+	}
+
 	free(gd);
 }
 
 static void *glx_decouple_user_data(backend_t *base attr_unused, void *ud attr_unused) {
-	auto ret = cmalloc(struct _glx_pixmap);
-	ret->owned = false;
-	ret->glpixmap = 0;
-	ret->pixmap = 0;
-	return ret;
+	return NULL;
 }
 
 static bool glx_set_swap_interval(int interval, Display *dpy, GLXDrawable drawable) {
@@ -224,6 +217,7 @@ static bool glx_set_swap_interval(int interval, Display *dpy, GLXDrawable drawab
 	return vsync_enabled;
 }
 
+const struct backend_operations glx_ops;
 /**
  * Initialize OpenGL.
  */
@@ -232,6 +226,7 @@ static backend_t *glx_init(session_t *ps, xcb_window_t target) {
 	glxext_init(ps->c.dpy, ps->c.screen);
 	auto gd = ccalloc(1, struct _glx_data);
 	init_backend_base(&gd->gl.base, ps);
+	gd->gl.base.ops = glx_ops;
 
 	gd->target_win = target;
 
@@ -259,11 +254,6 @@ static backend_t *glx_init(session_t *ps, xcb_window_t target) {
 		goto end;
 	}
 
-	if (glXGetConfig(ps->c.dpy, pvis, GLX_STENCIL_SIZE, &value) || !value) {
-		log_error("Root visual lacks stencil buffer.");
-		goto end;
-	}
-
 	if (glXGetConfig(ps->c.dpy, pvis, GLX_DOUBLEBUFFER, &value) || !value) {
 		log_error("Root visual is not a double buffered GL visual.");
 		goto end;
@@ -364,15 +354,10 @@ end:
 	return &gd->gl.base;
 }
 
-static void *
-glx_bind_pixmap(backend_t *base, xcb_pixmap_t pixmap, struct xvisual_info fmt, bool owned) {
-	struct _glx_pixmap *glxpixmap = NULL;
-	// Retrieve pixmap parameters, if they aren't provided
-	if (fmt.visual_depth > OPENGL_MAX_DEPTH) {
-		log_error("Requested depth %d higher than max possible depth %d.",
-		          fmt.visual_depth, OPENGL_MAX_DEPTH);
-		return NULL;
-	}
+static image_handle
+glx_bind_pixmap(backend_t *base, xcb_pixmap_t pixmap, struct xvisual_info fmt) {
+	GLXPixmap *glxpixmap = NULL;
+	auto gd = (struct _glx_data *)base;
 
 	if (fmt.visual_depth < 0) {
 		log_error("Pixmap %#010x with invalid depth %d", pixmap, fmt.visual_depth);
@@ -387,81 +372,87 @@ glx_bind_pixmap(backend_t *base, xcb_pixmap_t pixmap, struct xvisual_info fmt, b
 	}
 
 	log_trace("Binding pixmap %#010x", pixmap);
-	auto wd = default_new_backend_image(r->width, r->height);
 	auto inner = ccalloc(1, struct gl_texture);
 	inner->width = r->width;
 	inner->height = r->height;
-	wd->inner = (struct backend_image_inner_base *)inner;
+	inner->format = BACKEND_IMAGE_FORMAT_PIXMAP;
 	free(r);
 
-	auto fbcfg = glx_find_fbconfig(base->c, fmt);
-	if (!fbcfg) {
-		log_error("Couldn't find FBConfig with requested visual %x", fmt.visual);
-		goto err;
+	struct glx_fbconfig_cache *cached_fbconfig = NULL;
+	HASH_FIND(hh, gd->cached_fbconfigs, &fmt, sizeof(fmt), cached_fbconfig);
+	if (!cached_fbconfig) {
+		struct glx_fbconfig_info fbconfig;
+		if (!glx_find_fbconfig(base->c, fmt, &fbconfig)) {
+			log_error("Couldn't find FBConfig with requested visual %#x",
+			          fmt.visual);
+			goto err;
+		}
+		cached_fbconfig = cmalloc(struct glx_fbconfig_cache);
+		cached_fbconfig->visual_info = fmt;
+		cached_fbconfig->info = fbconfig;
+		HASH_ADD(hh, gd->cached_fbconfigs, visual_info, sizeof(fmt), cached_fbconfig);
+	} else {
+		log_debug("Found cached FBConfig for RGBA%d%d%d%d, depth: %d, visual id: "
+		          "%#x",
+		          fmt.red_size, fmt.blue_size, fmt.green_size, fmt.alpha_size,
+		          fmt.visual_depth, fmt.visual);
 	}
+	struct glx_fbconfig_info *fbconfig = &cached_fbconfig->info;
 
 	// Choose a suitable texture target for our pixmap.
 	// Refer to GLX_EXT_texture_om_pixmap spec to see what are the mean
 	// of the bits in texture_tgts
-	if (!(fbcfg->texture_tgts & GLX_TEXTURE_2D_BIT_EXT)) {
+	if (!(fbconfig->texture_tgts & GLX_TEXTURE_2D_BIT_EXT)) {
 		log_error("Cannot bind pixmap to GL_TEXTURE_2D, giving up");
 		goto err;
 	}
 
 	log_debug("depth %d, rgba %d", fmt.visual_depth,
-	          (fbcfg->texture_fmt == GLX_TEXTURE_FORMAT_RGBA_EXT));
+	          (fbconfig->texture_fmt == GLX_TEXTURE_FORMAT_RGBA_EXT));
 
 	GLint attrs[] = {
 	    GLX_TEXTURE_FORMAT_EXT,
-	    fbcfg->texture_fmt,
+	    fbconfig->texture_fmt,
 	    GLX_TEXTURE_TARGET_EXT,
 	    GLX_TEXTURE_2D_EXT,
 	    0,
 	};
 
-	inner->y_inverted = fbcfg->y_inverted;
+	inner->y_inverted = fbconfig->y_inverted;
 
-	glxpixmap = cmalloc(struct _glx_pixmap);
-	glxpixmap->pixmap = pixmap;
-	glxpixmap->glpixmap = glXCreatePixmap(base->c->dpy, fbcfg->cfg, pixmap, attrs);
-	glxpixmap->owned = owned;
-	free(fbcfg);
+	glxpixmap = cmalloc(GLXPixmap);
+	inner->pixmap = pixmap;
+	*glxpixmap = glXCreatePixmap(base->c->dpy, fbconfig->cfg, pixmap, attrs);
 
-	if (!glxpixmap->glpixmap) {
+	if (!*glxpixmap) {
 		log_error("Failed to create glpixmap for pixmap %#010x", pixmap);
 		goto err;
 	}
 
-	log_trace("GLXPixmap %#010lx", glxpixmap->glpixmap);
+	log_trace("GLXPixmap %#010lx", *glxpixmap);
 
 	// Create texture
 	inner->user_data = glxpixmap;
-	inner->texture = gl_new_texture(GL_TEXTURE_2D);
-	inner->has_alpha = fmt.alpha_size != 0;
-	wd->inner->refcount = 1;
+	inner->texture = gl_new_texture();
 	glBindTexture(GL_TEXTURE_2D, inner->texture);
-	glXBindTexImageEXT(base->c->dpy, glxpixmap->glpixmap, GLX_FRONT_LEFT_EXT, NULL);
+	glXBindTexImageEXT(base->c->dpy, *glxpixmap, GLX_FRONT_LEFT_EXT, NULL);
 	glBindTexture(GL_TEXTURE_2D, 0);
 
 	gl_check_err();
-	return wd;
+	return (image_handle)inner;
 err:
-	if (glxpixmap && glxpixmap->glpixmap) {
-		glXDestroyPixmap(base->c->dpy, glxpixmap->glpixmap);
+	if (glxpixmap && *glxpixmap) {
+		glXDestroyPixmap(base->c->dpy, *glxpixmap);
 	}
 	free(glxpixmap);
-
-	if (owned) {
-		xcb_free_pixmap(base->c->c, pixmap);
-	}
-	free(wd);
 	return NULL;
 }
 
-static void glx_present(backend_t *base, const region_t *region attr_unused) {
+static bool glx_present(backend_t *base) {
 	struct _glx_data *gd = (void *)base;
-	gl_present(base, region);
+	gl_finish_render(&gd->gl);
 	glXSwapBuffers(base->c->dpy, gd->target_win);
+	return true;
 }
 
 static int glx_buffer_age(backend_t *base) {
@@ -513,27 +504,41 @@ static void glx_diagnostics(backend_t *base) {
 	}
 }
 
-struct backend_operations glx_ops = {
+static int glx_max_buffer_age(struct backend_base *base attr_unused) {
+	return 5;        // Why?
+}
+
+#define PICOM_BACKEND_GLX_MAJOR (0UL)
+#define PICOM_BACKEND_GLX_MINOR (1UL)
+
+static void glx_version(struct backend_base * /*base*/, uint64_t *major, uint64_t *minor) {
+	*major = PICOM_BACKEND_GLX_MAJOR;
+	*minor = PICOM_BACKEND_GLX_MINOR;
+}
+
+const struct backend_operations glx_ops = {
+    .apply_alpha = gl_apply_alpha,
+    .back_buffer = gl_back_buffer,
+    .bind_pixmap = glx_bind_pixmap,
+    .blit = gl_blit,
+    .blur = gl_blur,
+    .clear = gl_clear,
+    .copy_area = gl_copy_area,
+    .copy_area_quantize = gl_copy_area_quantize,
+    .image_capabilities = gl_image_capabilities,
+    .is_format_supported = gl_is_format_supported,
+    .new_image = gl_new_image,
+    .present = glx_present,
+    .quirks = backend_no_quirks,
+    .version = glx_version,
+    .release_image = gl_release_image,
+
     .init = glx_init,
     .deinit = glx_deinit,
-    .bind_pixmap = glx_bind_pixmap,
-    .release_image = gl_release_image,
+    .root_change = gl_root_change,
     .prepare = gl_prepare,
-    .compose = gl_compose,
-    .image_op = gl_image_op,
-    .set_image_property = gl_set_image_property,
-    .clone_image = default_clone_image,
-    .blur = gl_blur,
-    .is_image_transparent = default_is_image_transparent,
-    .present = glx_present,
     .buffer_age = glx_buffer_age,
     .last_render_time = gl_last_render_time,
-    .create_shadow_context = gl_create_shadow_context,
-    .destroy_shadow_context = gl_destroy_shadow_context,
-    .render_shadow = backend_render_shadow_from_mask,
-    .shadow_from_mask = gl_shadow_from_mask,
-    .make_mask = gl_make_mask,
-    .fill = gl_fill,
     .create_blur_context = gl_create_blur_context,
     .destroy_blur_context = gl_destroy_blur_context,
     .get_blur_size = gl_get_blur_size,
@@ -542,7 +547,7 @@ struct backend_operations glx_ops = {
     .create_shader = gl_create_window_shader,
     .destroy_shader = gl_destroy_window_shader,
     .get_shader_attributes = gl_get_shader_attributes,
-    .max_buffer_age = 5,        // Why?
+    .max_buffer_age = glx_max_buffer_age,
 };
 
 struct glxext_info glxext = {0};
@@ -570,3 +575,10 @@ void glxext_init(Display *dpy, int screen) {
 #endif
 #undef check_ext
 }
+
+BACKEND_ENTRYPOINT(glx_register) {
+	if (!backend_register(PICOM_BACKEND_MAJOR, PICOM_BACKEND_MINOR, "glx",
+	                      glx_ops.init, true)) {
+		log_error("Failed to register glx backend");
+	}
+}
