@@ -1,15 +1,13 @@
 // SPDX-License-Identifier: MPL-2.0
 // Copyright (c) Yuxuan Shui <yshuiv7@gmail.com>
 #pragma once
+#include <stdbool.h>
+
 #include <X11/Xlib.h>
 #include <epoxy/glx.h>
-#include <stdbool.h>
 #include <xcb/render.h>
 #include <xcb/xcb.h>
 
-#include "compiler.h"
-#include "log.h"
-#include "utils.h"
 #include "x.h"
 
 struct glx_fbconfig_info {
@@ -19,21 +17,8 @@ struct glx_fbconfig_info {
 	int y_inverted;
 };
 
-/// The search criteria for glx_find_fbconfig
-struct glx_fbconfig_criteria {
-	/// Bit width of the red component
-	int red_size;
-	/// Bit width of the green component
-	int green_size;
-	/// Bit width of the blue component
-	int blue_size;
-	/// Bit width of the alpha component
-	int alpha_size;
-	/// The depth of X visual
-	int visual_depth;
-};
-
-struct glx_fbconfig_info *glx_find_fbconfig(struct x_connection *, struct xvisual_info);
+bool glx_find_fbconfig(struct x_connection *c, struct xvisual_info m,
+                       struct glx_fbconfig_info *info);
 
 struct glxext_info {
 	bool initialized;
