@@ -1,33 +1,42 @@
 #include "gl_common.h"
-
+// Don't macro expand bool!
+#undef bool
 // clang-format off
-const char dummy_frag[] = GLSL(330,
+const char copy_area_frag[] = GLSL(330,
+	layout(location = UNIFORM_TEX_LOC)
 	uniform sampler2D tex;
 	in vec2 texcoord;
 	void main() {
-		gl_FragColor = texelFetch(tex, ivec2(texcoord.xy), 0);
+		vec2 texsize = textureSize(tex, 0);
+		gl_FragColor = texture2D(tex, texcoord / texsize, 0);
 	}
 );
 
-const char present_frag[] = GLSL(330,
+const char copy_area_with_dither_frag[] = GLSL(330,
+	layout(location = UNIFORM_TEX_LOC)
 	uniform sampler2D tex;
 	in vec2 texcoord;
 	vec4 dither(vec4, vec2);
 	void main() {
-		gl_FragColor = dither(texelFetch(tex, ivec2(texcoord.xy), 0), texcoord);
+		vec2 texsize = textureSize(tex, 0);
+		gl_FragColor = dither(texture2D(tex, texcoord / texsize, 0), gl_FragCoord.xy);
 	}
 );
 
-const char copy_with_mask_frag[] = GLSL(330,
+const char blend_with_mask_frag[] = GLSL(330,
+	layout(location = UNIFORM_TEX_LOC)
 	uniform sampler2D tex;
+	layout(location = UNIFORM_OPACITY_LOC)
+	uniform float opacity;
 	in vec2 texcoord;
 	float mask_factor();
 	void main() {
-		gl_FragColor = texelFetch(tex, ivec2(texcoord.xy), 0) * mask_factor();
+		gl_FragColor = texelFetch(tex, ivec2(texcoord.xy), 0) * opacity * mask_factor();
 	}
 );
 
 const char fill_frag[] = GLSL(330,
+	layout(location = UNIFORM_COLOR_LOC)
 	uniform vec4 color;
 	void main() {
 		gl_FragColor = color;
@@ -36,6 +45,7 @@ const char fill_frag[] = GLSL(330,
 
 const char fill_vert[] = GLSL(330,
 	layout(location = 0) in vec2 in_coord;
+	layout(location = UNIFORM_PROJECTION_LOC)
 	uniform mat4 projection;
 	void main() {
 		gl_Position = projection * vec4(in_coord, 0, 1);
@@ -43,6 +53,7 @@ const char fill_vert[] = GLSL(330,
 );
 
 const char interpolating_frag[] = GLSL(330,
+	layout(location = UNIFORM_TEX_LOC)
 	uniform sampler2D tex;
 	in vec2 texcoord;
 	void main() {
@@ -51,7 +62,9 @@ const char interpolating_frag[] = GLSL(330,
 );
 
 const char interpolating_vert[] = GLSL(330,
+	layout(location = UNIFORM_PROJECTION_LOC)
 	uniform mat4 projection;
+	layout(location = UNIFORM_TEXSIZE_LOC)
 	uniform vec2 texsize;
 	layout(location = 0) in vec2 in_coord;
 	layout(location = 1) in vec2 in_texcoord;
@@ -62,14 +75,20 @@ const char interpolating_vert[] = GLSL(330,
 	}
 );
 const char masking_glsl[] = GLSL(330,
+	layout(location = UNIFORM_MASK_TEX_LOC)
 	uniform sampler2D mask_tex;
+	layout(location = UNIFORM_MASK_OFFSET_LOC)
 	uniform vec2 mask_offset;
+	layout(location = UNIFORM_MASK_CORNER_RADIUS_LOC)
 	uniform float mask_corner_radius;
+	layout(location = UNIFORM_MASK_INVERTED_LOC)
 	uniform bool mask_inverted;
 	in vec2 texcoord;
-	float mask_rectangle_sdf(vec2 point, vec2 half_size) {
-		vec2 d = abs(point) - half_size;
-		return length(max(d, 0.0));
+	vec2 mask_rectangle_sdf(vec2 point, vec2 half_size) {
+		vec2 d = max(abs(point) - half_size, 0.0);
+		float l = length(d);
+		// Add a small number to avoid 0/0.
+		return vec2(l, l / (max(d.x, d.y) + 1e-8));
 	}
 	float mask_factor() {
 		vec2 mask_size = textureSize(mask_tex, 0);
@@ -77,10 +96,11 @@ const char masking_glsl[] = GLSL(330,
 		vec4 mask = texture2D(mask_tex, maskcoord / mask_size);
 		if (mask_corner_radius != 0) {
 			vec2 inner_size = mask_size - vec2(mask_corner_radius) * 2.0f;
-			float dist = mask_rectangle_sdf(maskcoord - mask_size / 2.0f,
-			    inner_size / 2.0f) - mask_corner_radius;
+			vec2 sdf = mask_rectangle_sdf(maskcoord - mask_size / 2.0f,
+			    inner_size / 2.0f);
+			float dist = sdf.x - mask_corner_radius + sdf.y / 2.0f;
 			if (dist > 0.0f) {
-				mask.r *= (1.0f - clamp(dist, 0.0f, 1.0f));
+				mask.r *= (1.0f - clamp(dist, 0.0f, sdf.y) / (sdf.y + 1e-8));
 			}
 		}
 		if (mask_inverted) {
@@ -89,22 +109,37 @@ const char masking_glsl[] = GLSL(330,
 		return mask.r;
 	}
 );
-const char win_shader_glsl[] = GLSL(330,
+const char blit_shader_glsl[] = GLSL(330,
+	layout(location = UNIFORM_OPACITY_LOC)
 	uniform float opacity;
+	layout(location = UNIFORM_DIM_LOC)
 	uniform float dim;
+	layout(location = UNIFORM_CORNER_RADIUS_LOC)
 	uniform float corner_radius;
+	layout(location = UNIFORM_BORDER_WIDTH_LOC)
 	uniform float border_width;
+	layout(location = UNIFORM_INVERT_COLOR_LOC)
 	uniform bool invert_color;
 	in vec2 texcoord;
+	layout(location = UNIFORM_TEX_LOC)
 	uniform sampler2D tex;
+	layout(location = UNIFORM_EFFECTIVE_SIZE_LOC)
 	uniform vec2 effective_size;
+	layout(location = UNIFORM_BRIGHTNESS_LOC)
 	uniform sampler2D brightness;
+	layout(location = UNIFORM_MAX_BRIGHTNESS_LOC)
 	uniform float max_brightness;
+	layout(location = UNIFORM_TIME_LOC)
+	uniform float time;
 	// Signed distance field for rectangle center at (0, 0), with size of
 	// half_size * 2
-	float rectangle_sdf(vec2 point, vec2 half_size) {
-		vec2 d = abs(point) - half_size;
-		return length(max(d, 0.0));
+	// Returns 2 number: the distance, and the approximate chord length inside
+	// the pixel around `point`.
+	vec2 rectangle_sdf(vec2 point, vec2 half_size) {
+		vec2 d = max(abs(point) - half_size, 0.0);
+		float l = length(d);
+		// Add a small number to avoid 0/0.
+		return vec2(l, l / (max(d.x, d.y) + 1e-8));
 	}
 
 	vec4 default_post_processing(vec4 c) {
@@ -126,20 +161,30 @@ const char win_shader_glsl[] = GLSL(330,
 			border_color.rgb = border_color.rgb * (max_brightness / brightness);
 		}
 
-		// Rim color is the color of the outer rim of the window, if there is no
-		// border, it's the color of the window itself, otherwise it's the border.
-		// Using mix() to avoid a branch here.
-		vec4 rim_color = mix(c, border_color, clamp(border_width, 0.0f, 1.0f));
+		if (corner_radius != 0) {
+			// Rim color is the color of the outer rim of the window, if there is no
+			// border, it's the color of the window itself, otherwise it's the border.
+			// Using mix() to avoid a branch here.
+			vec4 rim_color = mix(c, border_color, clamp(border_width, 0.0f, 1.0f));
 
-		vec2 outer_size = effective_size;
-		vec2 inner_size = outer_size - vec2(corner_radius) * 2.0f;
-		float rect_distance = rectangle_sdf(texcoord - outer_size / 2.0f,
-		    inner_size / 2.0f) - corner_radius;
-		if (rect_distance > 0.0f) {
-			c = (1.0f - clamp(rect_distance, 0.0f, 1.0f)) * rim_color;
-		} else {
-			float factor = clamp(rect_distance + border_width, 0.0f, 1.0f);
-			c = (1.0f - factor) * c + factor * border_color;
+			vec2 outer_size = effective_size;
+			vec2 inner_size = outer_size - vec2(corner_radius) * 2.0f;
+			vec2 sdf = rectangle_sdf(texcoord - outer_size / 2.0f,
+			    inner_size / 2.0f);
+			// For anti-aliasing, we estimate how much of the pixel is covered by the rounded
+			// rectangle. This differs depends on at what angle the circle sweeps through the
+			// pixel. e.g. if it goes from corner to corner, then the coverage goes from 0 to
+			// 1 when the distance goes from -sqrt(2)/2 to +sqrt(2)/2; if it goes from egde to
+			// edge, then the coverage goes from 0 to 1 when the distance goes from -0.5 to 0.5.
+			// The chord length returned by `rectangle_sdf` is an approximation of this.
+			float rect_distance = sdf.x - corner_radius + sdf.y / 2.0f;
+			// Add a small number to sdf.y to avoid 0/0
+			if (rect_distance > 0.0f) {
+				c = (1.0f - clamp(rect_distance, 0.0f, sdf.y) / (sdf.y + 1e-8)) * rim_color;
+			} else {
+				float factor = clamp(rect_distance + border_width, 0.0f, sdf.y) / (sdf.y + 1e-8);
+				c = (1.0f - factor) * c + factor * border_color;
+			}
 		}
 
 		return c;
@@ -153,7 +198,7 @@ const char win_shader_glsl[] = GLSL(330,
 	}
 );
 
-const char win_shader_default[] = GLSL(330,
+const char blit_shader_default[] = GLSL(330,
 	in vec2 texcoord;
 	uniform sampler2D tex;
 	vec4 default_post_processing(vec4 c);
@@ -164,27 +209,20 @@ const char win_shader_default[] = GLSL(330,
 	}
 );
 
-const char present_vertex_shader[] = GLSL(330,
-	uniform mat4 projection;
-	layout(location = 0) in vec2 coord;
-	out vec2 texcoord;
-	void main() {
-		gl_Position = projection * vec4(coord, 0, 1);
-		texcoord = coord;
-	}
-);
 const char vertex_shader[] = GLSL(330,
+	layout(location = UNIFORM_PROJECTION_LOC)
 	uniform mat4 projection;
-	uniform float scale = 1.0;
-	uniform vec2 texorig;
+	layout(location = UNIFORM_SCALE_LOC)
+	uniform float scale = 1.0f;
 	layout(location = 0) in vec2 coord;
 	layout(location = 1) in vec2 in_texcoord;
 	out vec2 texcoord;
 	void main() {
 		gl_Position = projection * vec4(coord, 0, scale);
-		texcoord = in_texcoord + texorig;
+		texcoord = in_texcoord;
 	}
 );
+/// Add dithering for downsampling from 16-bit color to 8-bit color.
 const char dither_glsl[] = GLSL(330,
 	// Stolen from: https://www.shadertoy.com/view/7sfXDn
 	float bayer2(vec2 a) {
@@ -209,14 +247,4 @@ const char dither_glsl[] = GLSL(330,
 		return vec4(c + dithered * bayer(coord) / 255.0);
 	}
 );
-const char shadow_colorization_frag[] = GLSL(330,
-	uniform vec4 color;
-	uniform sampler2D tex;
-	in vec2 texcoord;
-	out vec4 out_color;
-	void main() {
-		vec4 c = texelFetch(tex, ivec2(texcoord), 0);
-		out_color = c.r * color;
-	}
-);
 // clang-format on
