@@ -1,15 +1,14 @@
 // SPDX-License-Identifier: MIT
 // Copyright (c) 2011-2013, Christopher Jeffrey
 // Copyright (c) 2013 Richard Grenville <pyxlcy@gmail.com>
+// Copyright (c) 2018 Yuxuan Shui <yshuiv7@gmail.com>
 
 #include <ctype.h>
-#include <errno.h>
+#include <dlfcn.h>
 #include <fcntl.h>
 #include <limits.h>
-#include <math.h>
 #include <stdbool.h>
 #include <stddef.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -18,21 +17,19 @@
 #include <unistd.h>
 #include <xcb/render.h>        // for xcb_render_fixed_t, XXX
 
+#include <picom/types.h>
 #include <test.h>
 
-#include "c2.h"
 #include "common.h"
-#include "compiler.h"
-#include "kernel.h"
 #include "log.h"
-#include "region.h"
-#include "string_utils.h"
-#include "types.h"
-#include "utils.h"
-#include "win.h"
+#include "utils/dynarr.h"
+#include "utils/kernel.h"
+#include "utils/str.h"
 
 #include "config.h"
 
+struct debug_options global_debug_options;
+
 const char *xdg_config_home(void) {
 	char *xdgh = getenv("XDG_CONFIG_HOME");
 	char *home = getenv("HOME");
@@ -43,10 +40,7 @@ const char *xdg_config_home(void) {
 			return NULL;
 		}
 
-		xdgh = cvalloc(strlen(home) + strlen(default_dir) + 1);
-
-		strcpy(xdgh, home);
-		strcat(xdgh, default_dir);
+		xdgh = mstrjoin(home, default_dir);
 	} else {
 		xdgh = strdup(xdgh);
 	}
@@ -184,7 +178,7 @@ const char *parse_readnum(const char *src, double *dest) {
 	return pc;
 }
 
-enum blur_method parse_blur_method(const char *src) {
+int parse_blur_method(const char *src) {
 	if (strcmp(src, "box") == 0) {
 		return BLUR_METHOD_BOX;
 	}
@@ -210,9 +204,8 @@ enum blur_method parse_blur_method(const char *src) {
  * @param[out] endptr return where the end of kernel is in the string
  * @param[out] hasneg whether the kernel has negative values
  */
-conv *parse_blur_kern(const char *src, const char **endptr, bool *hasneg) {
+static conv *parse_blur_kern(const char *src, const char **endptr) {
 	int width = 0, height = 0;
-	*hasneg = false;
 
 	const char *pc = NULL;
 
@@ -258,9 +251,6 @@ conv *parse_blur_kern(const char *src, const char **endptr, bool *hasneg) {
 			goto err2;
 		}
 		src = pc;
-		if (val < 0) {
-			*hasneg = true;
-		}
 		matrix->data[i] = val;
 	}
 
@@ -308,7 +298,7 @@ err1:
  * @param[out] hasneg whether any of the kernels have negative values
  * @return            the kernels
  */
-struct conv **parse_blur_kern_lst(const char *src, bool *hasneg, int *count) {
+struct conv **parse_blur_kern_lst(const char *src, int *count) {
 	// TODO(yshui) just return a predefined kernels, not parse predefined strings...
 	static const struct {
 		const char *name;
@@ -364,11 +354,10 @@ struct conv **parse_blur_kern_lst(const char *src, bool *hasneg, int *count) {
 	};
 
 	*count = 0;
-	*hasneg = false;
 	for (unsigned int i = 0;
 	     i < sizeof(CONV_KERN_PREDEF) / sizeof(CONV_KERN_PREDEF[0]); ++i) {
 		if (!strcmp(CONV_KERN_PREDEF[i].name, src)) {
-			return parse_blur_kern_lst(CONV_KERN_PREDEF[i].kern_str, hasneg, count);
+			return parse_blur_kern_lst(CONV_KERN_PREDEF[i].kern_str, count);
 		}
 	}
 
@@ -387,9 +376,8 @@ struct conv **parse_blur_kern_lst(const char *src, bool *hasneg, int *count) {
 	// Continue parsing until the end of source string
 	i = 0;
 	while (pc && *pc) {
-		bool tmp_hasneg;
 		assert(i < nkernels);
-		ret[i] = parse_blur_kern(pc, &pc, &tmp_hasneg);
+		ret[i] = parse_blur_kern(pc, &pc);
 		if (!ret[i]) {
 			for (int j = 0; j < i; j++) {
 				free(ret[j]);
@@ -398,7 +386,6 @@ struct conv **parse_blur_kern_lst(const char *src, bool *hasneg, int *count) {
 			return NULL;
 		}
 		i++;
-		*hasneg |= tmp_hasneg;
 	}
 
 	if (i > 1) {
@@ -413,113 +400,11 @@ struct conv **parse_blur_kern_lst(const char *src, bool *hasneg, int *count) {
 	return ret;
 }
 
-/**
- * Parse a X geometry.
- *
- * ps->root_width and ps->root_height must be valid
- */
-bool parse_geometry(session_t *ps, const char *src, region_t *dest) {
-	pixman_region32_clear(dest);
+void *parse_numeric_prefix(const char *src, const char **end, void *user_data) {
+	int *minmax = user_data;
+	*end = NULL;
 	if (!src) {
-		return true;
-	}
-	if (!ps->root_width || !ps->root_height) {
-		return true;
-	}
-
-	long x = 0, y = 0;
-	long width = ps->root_width, height = ps->root_height;
-	long val = 0L;
-	char *endptr = NULL;
-
-	src = skip_space(src);
-	if (!*src) {
-		goto parse_geometry_end;
-	}
-
-	// Parse width
-	// Must be base 10, because "0x0..." may appear
-	if (*src != '+' && *src != '-') {
-		val = strtol(src, &endptr, 10);
-		assert(endptr);
-		if (src != endptr) {
-			if (val < 0) {
-				log_error("Invalid width: %s", src);
-				return false;
-			}
-			width = val;
-			src = endptr;
-		}
-		src = skip_space(src);
-	}
-
-	// Parse height
-	if (*src == 'x') {
-		++src;
-		val = strtol(src, &endptr, 10);
-		assert(endptr);
-		if (src != endptr) {
-			if (val < 0) {
-				log_error("Invalid height: %s", src);
-				return false;
-			}
-			height = val;
-			src = endptr;
-		}
-		src = skip_space(src);
-	}
-
-	// Parse x
-	if (*src == '+' || *src == '-') {
-		val = strtol(src, &endptr, 10);
-		if (endptr && src != endptr) {
-			x = val;
-			if (*src == '-') {
-				x += ps->root_width - width;
-			}
-			src = endptr;
-		}
-		src = skip_space(src);
-	}
-
-	// Parse y
-	if (*src == '+' || *src == '-') {
-		val = strtol(src, &endptr, 10);
-		if (endptr && src != endptr) {
-			y = val;
-			if (*src == '-') {
-				y += ps->root_height - height;
-			}
-			src = endptr;
-		}
-		src = skip_space(src);
-	}
-
-	if (*src) {
-		log_error("Trailing characters: %s", src);
-		return false;
-	}
-
-parse_geometry_end:
-	if (x < INT_MIN || x > INT_MAX || y < INT_MIN || y > INT_MAX) {
-		log_error("Geometry coordinates exceeded limits: %s", src);
-		return false;
-	}
-	if (width > UINT_MAX || height > UINT_MAX) {
-		// less than 0 is checked for earlier
-		log_error("Geometry size exceeded limits: %s", src);
-		return false;
-	}
-	pixman_region32_union_rect(dest, dest, (int)x, (int)y, (uint)width, (uint)height);
-	return true;
-}
-
-/**
- * Parse a list of window rules, prefixed with a number, separated by a ':'
- */
-bool parse_numeric_window_rule(c2_lptr_t **res, const char *src, long min, long max) {
-	if (!src) {
-		return false;
+		return NULL;
 	}
 
 	// Find numeric value
@@ -527,12 +412,12 @@ bool parse_numeric_window_rule(c2_lptr_t **res, const char *src, long min, long
 	long val = strtol(src, &endptr, 0);
 	if (!endptr || endptr == src) {
 		log_error("No number specified: %s", src);
-		return false;
+		return NULL;
 	}
 
-	if (val < min || val > max) {
-		log_error("Number not in range (%ld <= n <= %ld): %s", min, max, src);
-		return false;
+	if (val < minmax[0] || val > minmax[1]) {
+		log_error("Number not in range (%d <= n <= %d): %s", minmax[0], minmax[1], src);
+		return NULL;
 	}
 
 	// Skip over spaces
@@ -541,12 +426,13 @@ bool parse_numeric_window_rule(c2_lptr_t **res, const char *src, long min, long
 	}
 	if (':' != *endptr) {
 		log_error("Number separator (':') not found: %s", src);
-		return false;
+		return NULL;
 	}
 	++endptr;
 
-	// Parse pattern
-	return c2_parse(res, endptr, (void *)val);
+	*end = endptr;
+
+	return (void *)val;
 }
 
 /// Search for auxiliary file under a base directory
@@ -563,7 +449,7 @@ static char *locate_auxiliary_file_at(const char *base, const char *scope, const
 }
 
 /**
- * Get a path of an auxiliary file to read, could be a shader file, or any supplimenrary
+ * Get a path of an auxiliary file to read, could be a shader file, or any supplementary
  * file.
  *
  * Follows the XDG specification to search for the shader file in configuration locations.
@@ -632,8 +518,10 @@ const char *vblank_scheduler_str[] = {
 	[LAST_VBLANK_SCHEDULER] = NULL
 };
 static const struct debug_options_entry debug_options_entries[] = {
-    {"smart_frame_pacing", NULL,                 offsetof(struct debug_options, smart_frame_pacing)},
-    {"force_vblank_sched", vblank_scheduler_str, offsetof(struct debug_options, force_vblank_scheduler)},
+    {"always_rebind_pixmap" , NULL                , offsetof(struct debug_options, always_rebind_pixmap)},
+    {"smart_frame_pacing"   , NULL                , offsetof(struct debug_options, smart_frame_pacing)},
+    {"force_vblank_sched"   , vblank_scheduler_str, offsetof(struct debug_options, force_vblank_scheduler)},
+    {"consistent_buffer_age", NULL                , offsetof(struct debug_options, consistent_buffer_age)},
 };
 // clang-format on
 
@@ -695,25 +583,24 @@ void parse_debug_options(struct debug_options *debug_options) {
 	}
 }
 
-/**
- * Parse a list of window shader rules.
- */
-bool parse_rule_window_shader(c2_lptr_t **res, const char *src, const char *include_dir) {
+void *parse_window_shader_prefix(const char *src, const char **end, void *user_data) {
+	const char *include_dir = user_data;
+	*end = NULL;
 	if (!src) {
-		return false;
+		return NULL;
 	}
 
 	// Find custom shader terminator
 	const char *endptr = strchr(src, ':');
 	if (!endptr) {
 		log_error("Custom shader terminator not found: %s", src);
-		return false;
+		return NULL;
 	}
 
 	// Parse and create custom shader
 	scoped_charp untrimed_shader_source = strdup(src);
 	if (!untrimed_shader_source) {
-		return false;
+		return NULL;
 	}
 	auto source_end = strchr(untrimed_shader_source, ':');
 	*source_end = '\0';
@@ -728,90 +615,35 @@ bool parse_rule_window_shader(c2_lptr_t **res, const char *src, const char *incl
 		if (!shader_source) {
 			log_error("Custom shader file \"%s\" not found for rule: %s", tmp, src);
 			free(shader_source);
-			return false;
+			return NULL;
 		}
 	}
 
-	return c2_parse(res, ++endptr, (void *)shader_source);
+	*end = endptr + 1;
+	return shader_source;
+}
+void *parse_window_shader_prefix_with_cwd(const char *src, const char **end, void *) {
+	scoped_charp cwd = getcwd(NULL, 0);
+	return parse_window_shader_prefix(src, end, cwd);
 }
 
-/**
- * Add a pattern to a condition linked list.
- */
-bool condlst_add(c2_lptr_t **pcondlst, const char *pattern) {
-	if (!pattern) {
-		return false;
+bool load_plugin(const char *name, const char *include_dir) {
+	scoped_charp path = locate_auxiliary_file("plugins", optarg, include_dir);
+	void *handle = NULL;
+	if (!path) {
+		handle = dlopen(name, RTLD_LAZY);
+	} else {
+		log_debug("Plugin %s resolved to %s", name, path);
+		handle = dlopen(path, RTLD_LAZY);
 	}
-
-	if (!c2_parse(pcondlst, pattern, NULL)) {
-		exit(1);
-	}
-
-	return true;
+	return handle != NULL;
 }
 
-void set_default_winopts(options_t *opt, win_option_mask_t *mask, bool shadow_enable,
-                         bool fading_enable, bool blur_enable) {
-	// Apply default wintype options.
-	if (!mask[WINTYPE_DESKTOP].shadow) {
-		// Desktop windows are always drawn without shadow by default.
-		mask[WINTYPE_DESKTOP].shadow = true;
-		opt->wintype_option[WINTYPE_DESKTOP].shadow = false;
-	}
-
-	// Focused/unfocused state only apply to a few window types, all other windows
-	// are always considered focused.
-	const wintype_t nofocus_type[] = {WINTYPE_UNKNOWN, WINTYPE_NORMAL, WINTYPE_UTILITY};
-	for (unsigned long i = 0; i < ARR_SIZE(nofocus_type); i++) {
-		if (!mask[nofocus_type[i]].focus) {
-			mask[nofocus_type[i]].focus = true;
-			opt->wintype_option[nofocus_type[i]].focus = false;
-		}
-	}
-	for (unsigned long i = 0; i < NUM_WINTYPES; i++) {
-		if (!mask[i].shadow) {
-			mask[i].shadow = true;
-			opt->wintype_option[i].shadow = shadow_enable;
-		}
-		if (!mask[i].fade) {
-			mask[i].fade = true;
-			opt->wintype_option[i].fade = fading_enable;
-		}
-		if (!mask[i].focus) {
-			mask[i].focus = true;
-			opt->wintype_option[i].focus = true;
-		}
-		if (!mask[i].blur_background) {
-			mask[i].blur_background = true;
-			opt->wintype_option[i].blur_background = blur_enable;
-		}
-		if (!mask[i].full_shadow) {
-			mask[i].full_shadow = true;
-			opt->wintype_option[i].full_shadow = false;
-		}
-		if (!mask[i].redir_ignore) {
-			mask[i].redir_ignore = true;
-			opt->wintype_option[i].redir_ignore = false;
-		}
-		if (!mask[i].opacity) {
-			mask[i].opacity = true;
-			// Opacity is not set to a concrete number here because the
-			// opacity logic is complicated, and needs an "unset" state
-			opt->wintype_option[i].opacity = NAN;
-		}
-		if (!mask[i].clip_shadow_above) {
-			mask[i].clip_shadow_above = true;
-			opt->wintype_option[i].clip_shadow_above = false;
-		}
-	}
-}
-
-char *parse_config(options_t *opt, const char *config_file, bool *shadow_enable,
-                   bool *fading_enable, bool *hasneg, win_option_mask_t *winopt_mask) {
+bool parse_config(options_t *opt, const char *config_file) {
 	// clang-format off
 	*opt = (struct options){
-	    .backend = BKEND_XRENDER,
-	    .legacy_backends = false,
+	    .legacy_backend = BKEND_XRENDER,
+	    .use_legacy_backends = false,
 	    .glx_no_stencil = false,
 	    .mark_wmwin_focused = false,
 	    .mark_ovredir_focused = false,
@@ -826,9 +658,10 @@ char *parse_config(options_t *opt, const char *config_file, bool *shadow_enable,
 	    .benchmark = 0,
 	    .benchmark_wid = XCB_NONE,
 	    .logpath = NULL,
+	    .log_level = LOG_LEVEL_WARN,
 
 	    .use_damage = true,
-	    .no_frame_pacing = false,
+	    .frame_pacing = true,
 
 	    .shadow_red = 0.0,
 	    .shadow_green = 0.0,
@@ -882,21 +715,28 @@ char *parse_config(options_t *opt, const char *config_file, bool *shadow_enable,
 
 	    .track_leader = false,
 
-	    .rounded_corners_blacklist = NULL
+	    .rounded_corners_blacklist = NULL,
+
+	    .rules = NULL,
 	};
 	// clang-format on
 
-	char *ret = NULL;
-#ifdef CONFIG_LIBCONFIG
-	ret = parse_config_libconfig(opt, config_file, shadow_enable, fading_enable,
-	                             hasneg, winopt_mask);
-#else
-	(void)config_file;
-	(void)shadow_enable;
-	(void)fading_enable;
-	(void)hasneg;
-	(void)winopt_mask;
-#endif
-	parse_debug_options(&opt->debug_options);
-	return ret;
+	list_init_head(&opt->included_config_files);
+	list_init_head(&opt->unredir_if_possible_blacklist);
+	list_init_head(&opt->paint_blacklist);
+	list_init_head(&opt->shadow_blacklist);
+	list_init_head(&opt->shadow_clip_list);
+	list_init_head(&opt->fade_blacklist);
+	list_init_head(&opt->blur_background_blacklist);
+	list_init_head(&opt->invert_color_list);
+	list_init_head(&opt->window_shader_fg_rules);
+	list_init_head(&opt->opacity_rules);
+	list_init_head(&opt->rounded_corners_blacklist);
+	list_init_head(&opt->corner_radius_rules);
+	list_init_head(&opt->focus_blacklist);
+	list_init_head(&opt->transparent_clipping_blacklist);
+	list_init_head(&opt->rules);
+
+	opt->all_scripts = dynarr_new(struct script *, 4);
+	return parse_config_libconfig(opt, config_file);
 }
