@@ -1,3 +1,6 @@
+// SPDX-License-Identifier: MPL-2.0
+// Copyright (c) Yuxuan Shui <yshuiv7@gmail.com>
+
 #include <assert.h>
 #include <stdarg.h>
 #include <stddef.h>
@@ -10,13 +13,12 @@
 
 #ifdef CONFIG_OPENGL
 #include <epoxy/gl.h>
-#include "backend/gl/gl_common.h"
-#include "backend/gl/glx.h"
 #endif
 
 #include "compiler.h"
 #include "log.h"
-#include "utils.h"
+#include "utils/console.h"
+#include "utils/misc.h"
 
 thread_local struct log *tls_logger;
 
@@ -78,7 +80,7 @@ static attr_const const char *log_level_to_string(enum log_level level) {
 	}
 }
 
-enum log_level string_to_log_level(const char *str) {
+int string_to_log_level(const char *str) {
 	if (strcasecmp(str, "TRACE") == 0) {
 		return LOG_LEVEL_TRACE;
 	}
@@ -273,7 +275,6 @@ static void file_logger_destroy(struct log_target *tgt) {
 	free(tgt);
 }
 
-#define ANSI(x) "\033[" x "m"
 static const char *terminal_colorize_begin(enum log_level level) {
 	switch (level) {
 	case LOG_LEVEL_TRACE: return ANSI("30;2");
