@@ -7,10 +7,10 @@
 #include <stdbool.h>
 #include <xcb/render.h>        // for xcb_render_fixed_t
 
+#include <picom/types.h>
+
 #include "compiler.h"
 #include "config.h"
-#include "types.h"
-#include "win.h"        // for wintype_t
 
 typedef struct session session_t;
 
@@ -30,8 +30,9 @@ bool get_early_config(int argc, char *const *argv, char **config_file, bool *all
  * Returns:
  *   Whether configuration are processed successfully.
  */
-bool must_use get_cfg(options_t *opt, int argc, char *const *argv, bool shadow_enable,
-                      bool fading_enable, bool conv_kern_hasneg,
-                      win_option_mask_t *winopt_mask);
+bool must_use get_cfg(options_t *opt, int argc, char *const *argv);
+void options_postprocess_c2_lists(struct c2_state *state, struct x_connection *c,
+                                  struct options *option);
+void options_destroy(struct options *options);
 
 // vim: set noet sw=8 ts=8:
