@@ -1,9 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 import xcffib.xproto as xproto
 import xcffib
 import time
-from common import set_window_name, trigger_root_configure
+from common import set_window_name
 
 conn = xcffib.connect()
 setup = conn.get_setup()
